
/*
	broker.java
	written by: Dan Marzullo & Jakub Kolodziejski
	debugged by: Dan Marzullo & Jakub Kolodziejski
	
	Created: 10/14/2011
	Modified: 12/8/2011
	
	Main class file of "Broker Bob" program which handles the backend
	stock order processing and execution on thestockhop.com
*/

import java.io.*;
import java.util.*;
import java.math.*;
import java.sql.*;

public class broker 
{
	// Controls whether to print debug statements to console/shell
	public static boolean debug = true;

	public static void main(String[] args) throws InterruptedException
	{
		debug("STARTING!!!");
		
		// Order commissions
		double sellCommission = 10;
		double buyCommission = 20;
		
		Database database = new Database();
		
		// Connect to database
		database.connect("thestockhop","root","");
		
		// MAIN LOOP
		while(true)
		{
			debug("SLEEPING");
			Thread.sleep(3000);			
			// Fetch open orders
			database.readOrders();
		
			// Get list of stock symbols used in orders (duplicates avoided)
			String orderSymbols = database.getSymbols();
			debug("Order Symbols: " + orderSymbols);
			
			CSVParser p = new CSVParser();
			
			// Pass symbols to Parser for retrieval
			if (orderSymbols != "") 
			{
				p = new CSVParser(orderSymbols);
				p.Parse();
				
				
				// Get an order
				Order currentOrder = new Order();
				
				for(currentOrder = database.getOrder(); currentOrder != null; currentOrder = database.getOrder())
				{
					
					// *** TIME ELAPSED CHECK ***
					// Get the current time and test for 20 minutes passing (1200000 ms)
					java.util.Date date = new java.util.Date();
					Timestamp currentTime = new Timestamp(date.getTime());
					if (currentTime.getTime() - currentOrder.timePlaced.getTime() >= 10000)
					{
						// *** VOLUME CHECK ***
						debug("Volume: " +p.getVolume(currentOrder.symbol));
						if (currentOrder.quantity <= p.getVolume(currentOrder.symbol))
						{
							//------------------------------------------------------------------------------------------------------
							//*****************************************   Market Orders   ******************************************
							//------------------------------------------------------------------------------------------------------
							if (currentOrder.type.equals("Market"))
							{					
								debug(currentOrder.type + " - " + currentOrder.action);
								debug("Market Price: " +p.getPrice(currentOrder.symbol));
								if (currentOrder.action.equals("Sell")) 
								{
									executeSell(database, currentOrder, p, sellCommission, buyCommission);
								}
								else  //Buy
								{
									executeBuy(database, currentOrder, p, sellCommission, buyCommission);
								}
							}
							//------------------------------------------------------------------------------------------------------
							//*****************************************   Limit Orders   *******************************************
							//------------------------------------------------------------------------------------------------------
							else if (currentOrder.type.equals("Limit"))
							{
								debug(currentOrder.type + " - " + currentOrder.action);
								if (currentOrder.action.equals("Sell")) 
								{
									debug("Limit Price: " + currentOrder.limitPrice + " <= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if (currentOrder.limitPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) <= 0)
									{
										executeSell(database, currentOrder, p, sellCommission, buyCommission);
									}
									else { debug("ORDER NOT EXECUTED"); }
								}
								else  //Buy
								{
									debug("Limit Price: " + currentOrder.limitPrice + " >= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if (currentOrder.limitPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) >= 0)
									{
										executeBuy(database, currentOrder, p, sellCommission, buyCommission);
									}
									else { debug("ORDER NOT EXECUTED"); }
									
								}
							}
							//------------------------------------------------------------------------------------------------------
							//*****************************************   Stop Orders   ********************************************
							//------------------------------------------------------------------------------------------------------					
							else if (currentOrder.type.equals("Stop"))
							{
								debug(currentOrder.type + " - " + currentOrder.action);
								if (currentOrder.action.equals("Sell")) 
								{
									debug("Stop Price:  " + currentOrder.stopPrice + " >= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if (currentOrder.stopPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) >= 0)
									{
										executeSell(database, currentOrder, p, sellCommission, buyCommission);
									}
									else { debug("ORDER NOT EXECUTED"); }
								}
								else  //Buy
								{
									debug("Stop Price:  " + currentOrder.stopPrice + " <= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if (currentOrder.stopPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) <= 0)
									{
										executeBuy(database, currentOrder, p, sellCommission, buyCommission);
									}
									else { debug("ORDER NOT EXECUTED"); }
								}
							}
							//------------------------------------------------------------------------------------------------------
							//*****************************************   Stop Limit Orders   **************************************
							//------------------------------------------------------------------------------------------------------
							else if (currentOrder.type.equals("Stop-Limit"))
							{
								debug(currentOrder.type + " - " + currentOrder.action);
								if (currentOrder.action.equals("Sell")) 
								{
									debug("Stop Price: " + currentOrder.stopPrice + " >= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									debug("Limit Price: " + currentOrder.limitPrice + " <= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if ((currentOrder.stopPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) >= 0) && (currentOrder.limitPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) <= 0))
									{
										executeSell(database, currentOrder, p, sellCommission, buyCommission);
									}
									else { debug("ORDER NOT EXECUTED"); }
								}
								else  //Buy
								{
									debug("Stop Price: " + currentOrder.stopPrice + " <= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									debug("Limit Price: " + currentOrder.limitPrice + " >= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if ((currentOrder.stopPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) <= 0) && (currentOrder.limitPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) >= 0))
									{
										executeBuy(database, currentOrder, p, sellCommission, buyCommission);
									}
									else { debug("ORDER NOT EXECUTED"); }
								}
							}
							//------------------------------------------------------------------------------------------------------
							//*****************************************   Trailing Stop Orders   ***********************************
							//------------------------------------------------------------------------------------------------------					
							else if (currentOrder.type.equals("Trailing Stop"))
							{
								debug(currentOrder.type + " - " + currentOrder.action);
								if (currentOrder.action.equals("Sell")) 
								{
									debug("Stop Price: " + currentOrder.stopPrice + " >= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if (currentOrder.stopPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) >= 0)
									{
										executeSell(database, currentOrder, p, sellCommission, buyCommission);
									}
									else	// adjust Stop Price as needed
									{
										debug("ORDER NOT EXECUTED");
										debug("Adjusting stop price if need be...");
										// if market price rises, increase Stop Price to market price less trail percentage
										BigDecimal tmpMarketPrice = new BigDecimal(p.getPrice(currentOrder.symbol));
										BigDecimal tmpTrailAmount = currentOrder.trail.multiply(tmpMarketPrice);
										BigDecimal tmpStopPlusTrail = currentOrder.stopPrice.add(tmpTrailAmount);
										if (tmpStopPlusTrail.compareTo(tmpMarketPrice) < 0)
										{
											database.updateStopPrice(currentOrder.id, tmpMarketPrice.subtract(tmpTrailAmount));
											debug("Stop price adjusted");
										}
										// else do nothing
									}
								}
								else  //Buy
								{
									debug("Stop Price: " + currentOrder.stopPrice + " <= Market Price: " +p.getPrice(currentOrder.symbol) + " ???");
									if (currentOrder.stopPrice.compareTo(new BigDecimal(p.getPrice(currentOrder.symbol))) <= 0)
									{
										executeBuy(database, currentOrder, p, sellCommission, buyCommission);
									}
									else	// adjust Stop Price as needed
									{
										debug("ORDER NOT EXECUTED");
										debug("Adjusting stop price if need be...");
										// if market price falls, decrease Stop Price to market price plus trail percentage
										BigDecimal tmpMarketPrice = new BigDecimal(p.getPrice(currentOrder.symbol));
										BigDecimal tmpTrailAmount = currentOrder.trail.multiply(tmpMarketPrice);
										BigDecimal tmpStopLessTrail = currentOrder.stopPrice.subtract(tmpTrailAmount);
										if (tmpStopLessTrail.compareTo(tmpMarketPrice) > 0)
										{
											database.updateStopPrice(currentOrder.id, tmpMarketPrice.add(tmpTrailAmount));
											debug("Stop price adjusted");
										}
										// else do nothing
									}
									
								}
							}
						}
						else { debug("ORDER NOT EXECUTED: not enough volume"); }
					}
					else { debug("ORDER NOT EXECUTED: too soon"); }
				}
			}
			debug("FINISHED THIS ROUND!!!");
		}
	}
	
	// Executes a buy order
	public static void executeBuy(Database database, Order currentOrder, CSVParser p, double sellCommission, double buyCommission)
	{
		BigDecimal currentPrice = new BigDecimal(p.getPrice(currentOrder.symbol));
		BigDecimal quantity = new BigDecimal(currentOrder.quantity);
		BigDecimal cash = new BigDecimal(0);
		cash = database.getUserCash(currentOrder.account);
		
		if (cash.compareTo(currentPrice.multiply(quantity)) >= 0)
		{
			debug("EXECUTING...");
			database.addToPortfolio(currentOrder.account, currentOrder.symbol, currentOrder.quantity, currentPrice);
			cash = cash.subtract(currentPrice.multiply(quantity));
			cash = cash.subtract(new BigDecimal(buyCommission));
			database.setUserCash(currentOrder.account, cash);
			database.addHistory(currentOrder.account, currentOrder.timePlaced, currentOrder.symbol, currentOrder.action, currentOrder.quantity, currentOrder.type, currentOrder.stopPrice, currentOrder.limitPrice, currentOrder.trail, currentOrder.duration, currentPrice, new BigDecimal(sellCommission), currentPrice.multiply(quantity), database.getUserCash(currentOrder.account));
			database.deleteOrder(currentOrder.id);
			if(currentOrder.notify > 0 )
			{
				String to = database.getUserEmail(currentOrder.account);
				String message = "You have purchased " + currentOrder.quantity + " shares of " + currentOrder.symbol + " at $" + p.getPrice(currentOrder.symbol);
				Mail email = new Mail(to, message);
				email.send();
				if (database.getUserPhone(currentOrder.account) != null) {
					SMS text = new SMS(database.getUserSMS(currentOrder.account), message);
					text.send();
				}
			}
			debug("ORDER EXECUTED!!!");
		}
		else { debug("ORDER NOT EXECUTED: not enough cash"); }
	}
	
	// Executes a sell order
	public static void executeSell(Database database, Order currentOrder, CSVParser p, double sellCommission, double buyCommission)
	{
		BigDecimal currentPrice = new BigDecimal(p.getPrice(currentOrder.symbol));
		BigDecimal quantity = new BigDecimal(currentOrder.quantity);
		BigDecimal cash = new BigDecimal(0);
		cash = database.getUserCash(currentOrder.account);
		
		if (database.getPortfolioStock(currentOrder.account, currentOrder.symbol) >= currentOrder.quantity)
		{
			debug("EXECUTING...");
			database.removeFromPortfolio(currentOrder.account, currentOrder.symbol, currentOrder.quantity);
			cash = cash.add(currentPrice.multiply(quantity));
			cash = cash.subtract(new BigDecimal(sellCommission));
			database.setUserCash(currentOrder.account, cash);
			database.addHistory(currentOrder.account, currentOrder.timePlaced, currentOrder.symbol, currentOrder.action, currentOrder.quantity, currentOrder.type, currentOrder.stopPrice, currentOrder.limitPrice, currentOrder.trail, currentOrder.duration, currentPrice, new BigDecimal(sellCommission), currentPrice.multiply(quantity), database.getUserCash(currentOrder.account));
			database.deleteOrder(currentOrder.id);
			if(currentOrder.notify > 0 )
			{
				String to = database.getUserEmail(currentOrder.account);
				String message = "You have sold " + currentOrder.quantity + " shares of " + currentOrder.symbol + " at $" + p.getPrice(currentOrder.symbol);
				Mail email = new Mail(to, message);
				email.send();
				if (database.getUserPhone(currentOrder.account) != null) {
					SMS text = new SMS(database.getUserSMS(currentOrder.account), message);
					text.send();
				}
			}
			debug("ORDER EXECUTED!!!");
		}
		else { debug("ORDER NOT EXECUTED: not enough cash"); }
	}
	
	// Prints debug statements to console/shell.
	public static void debug(String text) {
		if (debug) {
			System.out.println("" + text);
		}
	}
}
