<?php
// written by:   Melissa Romanus
// assisted by:
// debugged by:

namespace RU\TheStockHopBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use RU\TheStockHopBundle\Entity\Account;
use RU\TheStockHopBundle\Entity\Order;
use RU\TheStockHopBundle\StockFacts;
use Symfony\Component\Form\FormError;

class TradeController extends Controller
{
        public function __construct()
        {
        }

        public function indexAction(Request $request)
        {
	        if ($this->get('security.context')->isGranted('IS_AUTHENTICATED_FULLY')) {
		    $user = $this->get('security.context')->getToken()->getUser();

		    $order = new Order();
		    $order->setNotify($user->getNotify());

            $order->setLimitPrice(0);
            $order->setStopPrice(0);
            $order->setTrail(0);


		    $form = $this->makeForm($order);

	            return $this->render('TheStockHopBundle:Trade:trade2.html.twig', array('form' => $form->createView(),));
	        } else {
	            return $this->render('TheStockHopBundle:Home:home_unauth.html.twig');
		}
        }
	
	public function validateAction(Request $request){

                if ($this->get('security.context')->isGranted('IS_AUTHENTICATED_FULLY')) {
                    $user = $this->get('security.context')->getToken()->getUser();
                    $order = new Order();

		    		$user->getOrders()->add($order);
                    $order->setAccount($user);


                    $form = $this->makeForm($order);
                    if ($request->getMethod() == 'POST') {
	                    $form->bindRequest($request);
			   
			    // Check and make sure we can afford this Order if type is buy
			    if( $order->getAction() == "Sell"){
			    	if( is_int($order->getQuantity()) && $order->getQuantity() > 0 && is_double($order->getLimitPrice()) && $order->getLimitPrice() > 0){
			    		$cost = $order->getQuantity() * $order->getLimitPrice();
					if ( $cost > $user->getCash() ){
						$form->addError(new FormError("You do not have enough cash to complete this order."));
					}
			    	}
			    }
			    $postData = $request->request->get('form');
			    // Remove fielfs from bvalidation that are not needed for this order type
			    if( $order->getType() == "Market"){
					$postData['limitPrice'] = 5;
					$postData['stopPrice'] = 5;
					$postData['trail'] = 5;
			    }

			    if( $order->getType() == "Limit"){
					$postData['stopPrice'] = 5;
					$postData['trail'] = 5;
			    }

			    if( $order->getType() == "Stop"){
					$postData['limitPrice'] = 5;
					$postData['trail'] = 5;
			    }

			    if( $order->getType() == "Stop-Limit"){
					$postData['trail'] = 5;
			    }

			    if( $order->getType() == "Trailing Stop"){
			    	$postData['limitPrice'] = 5;
			    	$postData['stopPrice'] = 5;
			    }
			    unset($form);
			    $form = $this->makeForm($order);
			    $form->bind($postData);

			    if( $form->hasErrors()) {
					$this->get("session")->setFlash('failure',"There is an error in the Order Form.");
			    }
			    if ($form->isValid() and !$form->hasErrors()){
                    $em = $this->getDoctrine()->getEntityManager();

				try{
					$em->persist($order);
					$em->flush();
		                        $order = new Order();
		                        $order->setLimitPrice(0);
		                        $order->setStopPrice(0);
		                        $order->setTrail(0);

		                        $form = $this->makeForm($order);
				
					$this->get("session")->setFlash('success', "Your Order was successfuly submitted.");

					return $this->render('TheStockHopBundle:Trade:trade2.html.twig', array('form' => $form->createView()));

				}catch(\Exception $e) {
					print $e;
					return "DATABASE ERROR ".$e;
				}

				return "this is valid";
			    }else{
			    	//$form = $this->makeForm($order);
					return $this->render('TheStockHopBundle:Trade:trade2.html.twig', array('form' => $form->createView()));
			    }if ( $form->isValid()){

			    }
		    }

                } else {
                    return $this->render('TheStockHopBundle:Home:home_unauth.html.twig');
                }

	}

	public function sellAction(Request $request){
		continue;
	}	

	private function makeForm($order){
		return  $this->createFormBuilder($order)
                        ->add('symbol', 'text')
                        ->add('action', 'choice', array('choices' => array('Buy' => 'Buy','Sell' => 'Sell')))
                        ->add('quantity', 'integer')
                        ->add('type', 'choice', array('choices' => array('Market' => 'Market','Limit' => 'Limit','Stop' => 'Stop','Stop-Limit' => 'Stop-Limit','Trailing Stop' => 'Trailing Stop')))
                        ->add('limitPrice', 'money', array('label' => 'Limit Price ', 'currency' => 'USD','required'  => false))
                        ->add('stopPrice', 'money', array('label' => 'Stop Price ', 'currency' => 'USD','required'  => false))
                        ->add('trail', 'integer', array('label' => 'Trail% ','required'  => false))
                        ->add('duration','choice',array('choices' => array('Day' => 'Good For Day', 'Cancel' => 'Good Until Canceled')))
                        ->add('notify','checkbox', array( 'label' => 'Notify me when order completes.', 'required'  => false))
                        ->getForm();
	}

}
