<?php
// written by:   Melissa Romanus
// assisted by:
// debugged by:

namespace RU\TheStockHopBundle\Entity;

use Symfony\Component\Security\Core\User\UserInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;


/**
 * @ORM\Entity
 * @ORM\Table(name="accounts")
 */
class Account implements UserInterface
{
        /** 
        *   @ORM\Id
	*   @ORM\Column(type="integer") 
	*   @ORM\GeneratedValue
	*/
        protected $id;

        /**
	*  @ORM\Column(name="user",length=255, unique="TRUE")
	*/
        protected $username;

	/**
	*  @ORM\Column(type="string", length="255")
	*/
	protected $password;

	/**
	* @ORM\Column(type="string", length="255")
	*
	* @var string salt
	*/
	protected $salt;

	/**
	* @ORM\ManyToMany(targetEntity="Role")
	* @ORM\JoinTable(name="account_role",
	*     joinColumns={@ORM\JoinColumn(name="user_id", referencedColumnName="id")},
	*     inverseJoinColumns={@ORM\JoinColumn(name="role_id", referencedColumnName="id")}
	* )
	*
	* @var ArrayCollection $userRoles
	*/
 	protected $userRoles;

        /**
        *  @ORM\Column(name="phone",length=10)
        */
        protected $phoneNumber;


        /**
        *  @ORM\Column(length=30)
        */
        protected $carrier;


	/**
	*  @ORM\Column(name="registered",type="datetime") 
	*/
	protected $registrationDate;

	/**
	*  @ORM\Column(length=64)
	*/
	protected $email;

	/**
	*  @ORM\Column(name="default_notify",type="boolean")
	*/
	protected $notify;

        /** @ORM\Column(type="decimal", scale=2) */
	protected $cash;

	/**
	*  @ORM\OneToMany(targetEntity="Order",mappedBy="account")
	*  @ORM\OrderBy({"created" = "ASC"})
	*/
	private $orders;


        /**
        *  @ORM\OneToMany(targetEntity="History",mappedBy="account")
        *  @ORM\OrderBy({"created" = "ASC"})
        */
        private $history;


        /**
        *  @ORM\OneToMany(targetEntity="Portfolio",mappedBy="account")
        */
        private $portfolio;


	public $confirmPassword;

	public $newPassword;

	public $oldPassword;

    /**
     * Get id
     *
     * @return integer 
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set username
     *
     * @param string $username
     */
    public function setUsername($username)
    {
        $this->username = $username;
    }

    /**
     * Get username
     *
     * @return string 
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * Set password
     *
     * @param string $password
     */
    public function setPassword($password)
    {
        $this->password = $password;
    }

    /**
     * Get password
     *
     * @return string 
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Set registrationDate
     *
     * @param datetime $registrationDate
     */
    public function setRegistrationDate($registrationDate)
    {
        $this->registrationDate = new \DateTime("now");
        #$this->registrationDate = $registrationDate;
    }

    /**
     * Get registrationDate
     *
     * @return datetime 
     */
    public function getRegistrationDate()
    {
        return $this->registrationDate;
    }

    /**
     * Set email
     *
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * Get email
     *
     * @return string 
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * Set default_notify
     *
     * @param boolean $defaultNotify
     */
    public function setDefaultNotify($defaultNotify)
    {
        $this->default_notify = $defaultNotify;
    }

    /**
     * Get default_notify
     *
     * @return boolean 
     */
    public function getDefaultNotify()
    {
        return $this->default_notify;
    }

    /**
     * Set cash
     *
     * @param decimal $cash
     */
    public function setCash($cash)
    {
        $this->cash = $cash;
    }

    /**
     * Get cash
     *
     * @return decimal 
     */
    public function getCash()
    {
        return $this->cash;
    }

    public function __construct()
    {
        $this->orders = new \Doctrine\Common\Collections\ArrayCollection();
	$this->history = new \Doctrine\Common\Collections\ArrayCollection();
	$this->portfolio = new \Doctrine\Common\Collections\ArrayCollection();
        $this->userRoles = new \Doctrine\Common\Collections\ArrayCollection();
    }
    
    /**
     * Set notify
     *
     * @param boolean $notify
     */
    public function setNotify($notify)
    {
        $this->notify = $notify;
    }

    /**
     * Get notify
     *
     * @return boolean 
     */
    public function getNotify()
    {
        return $this->notify;
    }

    /**
     * Add orders
     *
     * @param RU\TheStockHopBundle\Entity\Order $orders
     */
    public function addOrder(\RU\TheStockHopBundle\Entity\Order $orders)
    {
        $this->orders[] = $orders;
    }

    /**
     * Get orders
     *
     * @return Doctrine\Common\Collections\Collection 
     */
    public function getOrders()
    {
        return $this->orders;
    }

    /**
     * Add history
     *
     * @param RU\TheStockHopBundle\Entity\History $history
     */
    public function addHistory(\RU\TheStockHopBundle\Entity\History $history)
    {
        $this->history[] = $history;
    }

    /**
     * Get history
     *
     * @return Doctrine\Common\Collections\Collection 
     */
    public function getHistory()
    {
        return $this->history;
    }

    /**
     * Add portfolio
     *
     * @param RU\TheStockHopBundle\Entity\Portfolio $portfolio
     */
    public function addPortfolio(\RU\TheStockHopBundle\Entity\Portfolio $portfolio)
    {
        $this->portfolio[] = $portfolio;
    }

    /**
     * Get portfolio
     *
     * @return Doctrine\Common\Collections\Collection 
     */
    public function getPortfolio()
    {
        return $this->portfolio;
    }

    /**
     * Gets the user salt.
     *
     * @return string The salt.
     */
    public function getSalt()
    {
        return $this->salt;
    }
 
    /**
     * Sets the user salt.
     *
     * @param string $value The salt.
     */
    public function setSalt($value)
    {
        $this->salt = $value;
    }

    /**
     * Compares this user to another to determine if they are the same.
     * 
     * @param UserInterface $user The user
     * @return boolean True if equal, false othwerwise.
     */
    public function equals(UserInterface $user)
    {
        return md5($this->getUsername()) == md5($user->getUsername());
    }

    /**
     * Erases the user credentials.
     */
    public function eraseCredentials()
    {
 
    }
 
    /**
     * Gets an array of roles.
     * 
     * @return array An array of Role objects
     */
    public function getRoles()
    {
        return $this->getUserRoles()->toArray();
    }

    /**
     * Add userRoles
     *
     * @param RU\TheStockHopBundle\Entity\Role $userRoles
     */
    public function addRole(\RU\TheStockHopBundle\Entity\Role $userRoles)
    {
        $this->userRoles[] = $userRoles;
    }

    /**
     * Get userRoles
     *
     * @return Doctrine\Common\Collections\Collection 
     */
    public function getUserRoles()
    {
        return $this->userRoles;
    }

    /**
     * Set phoneNumber
     *
     * @param string $phoneNumber
     */
    public function setPhoneNumber($phoneNumber)
    {
        $this->phoneNumber = $phoneNumber;
    }

    /**
     * Get phoneNumber
     *
     * @return string 
     */
    public function getPhoneNumber()
    {
        return $this->phoneNumber;
    }

    /**
     * Set carrier
     *
     * @param string $carrier
     */
    public function setCarrier($carrier)
    {
        $this->carrier = $carrier;
    }

    /**
     * Get carrier
     *
     * @return string 
     */
    public function getCarrier()
    {
        return $this->carrier;
    }
}